searchData={"items":[{"type":"behaviour","title":"FatEcto.Dynamics.FatBuildable","doc":"Builds queries after filtering fields based on user-provided filterable and overrideable fields.\n\nThis module provides functionality to filter Ecto queries using predefined filterable fields (handled by `Builder`)\nand overrideable fields (handled by a fallback function).","ref":"FatEcto.Dynamics.FatBuildable.html"},{"type":"behaviour","title":"Options - FatEcto.Dynamics.FatBuildable","doc":"- `filterable_fields`: A map of fields and their allowed operators. Example:\n    %{\n      \"id\" => [\"$EQUAL\", \"$NOT_EQUAL\"],\n      \"name\" => [\"$ILIKE\"]\n    }\n- `overrideable_fields`: A list of fields that can be overridden. Example:\n    [\"name\", \"phone\"]\n- `ignoreable_fields_values`: A map of fields and their ignoreable values. Example:\n    %{\n      \"name\" => [\"%%\", \"\", [], nil],\n      \"phone\" => [\"%%\", \"\", [], nil]\n    }","ref":"FatEcto.Dynamics.FatBuildable.html#module-options"},{"type":"behaviour","title":"Example Usage - FatEcto.Dynamics.FatBuildable","doc":"defmodule FatEcto.Dynamics.MyApp.HospitalFilter do\n      use FatEcto.Dynamics.FatBuildable,\n        filterable_fields: %{\n          \"id\" => [\"$EQUAL\", \"$NOT_EQUAL\"]\n        },\n        overrideable_fields: [\"name\", \"phone\"],\n        ignoreable_fields_values: %{\n          \"name\" => [\"%%\", \"\", [], nil],\n          \"phone\" => [\"%%\", \"\", [], nil]\n        }\n\n      import Ecto.Query\n\n      def override_whereable(_dynamics, \"name\", \"$ILIKE\", value) do\n        dynamic([q], ilike(fragment(\"(?)::TEXT\", q.name), ^value))\n      end\n\n      def override_whereable(_dynamics, \"phone\", \"$ILIKE\", value) do\n        dynamic([q], ilike(fragment(\"(?)::TEXT\", q.phone), ^value))\n      end\n\n      def override_whereable(dynamics, _, _, _) do\n        dynamics\n      end\n\n      # Optional: Override after_whereable to perform custom processing on the final dynamics\n      def after_whereable(dynamics) do\n        IO.puts(\"Do something on final Dynamics\")\n        dynamics\n      end\n    end","ref":"FatEcto.Dynamics.FatBuildable.html#module-example-usage"},{"type":"callback","title":"FatEcto.Dynamics.FatBuildable.after_whereable/1","doc":"Callback for performing custom processing on the final dynamics.\n\nThis function is called at the end of the `build/2` function. The default behavior is to return the dynamics,\nbut it can be overridden by the using module.","ref":"FatEcto.Dynamics.FatBuildable.html#c:after_whereable/1"},{"type":"callback","title":"FatEcto.Dynamics.FatBuildable.override_whereable/4","doc":"Callback for handling custom filtering logic for overrideable fields.\n\nThis function acts as a fallback for overrideable fields. The default behavior is to return the dynamics,\nbut it can be overridden by the using module.","ref":"FatEcto.Dynamics.FatBuildable.html#c:override_whereable/4"},{"type":"module","title":"FatEcto.Dynamics.FatBuildableHelper","doc":"Provides helper functions for filtering and processing query parameters in `FatEcto.Dynamics.FatBuildable`.","ref":"FatEcto.Dynamics.FatBuildableHelper.html"},{"type":"function","title":"FatEcto.Dynamics.FatBuildableHelper.filter_filterable_fields/3","doc":"Filters fields based on the `filterable_fields` configuration.","ref":"FatEcto.Dynamics.FatBuildableHelper.html#filter_filterable_fields/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatBuildableHelper.filter_filterable_fields/3","doc":"- `where_params`: The query parameters (e.g., `%{\"field\" => %{\"$EQUAL\" => \"value\"}}`).\n  - `filterable_fields`: A map of fields and their allowed operators (e.g., `%{\"email\" => [\"$EQUAL\", \"$LIKE\"]}`).","ref":"FatEcto.Dynamics.FatBuildableHelper.html#filter_filterable_fields/3-parameters"},{"type":"function","title":"Returns - FatEcto.Dynamics.FatBuildableHelper.filter_filterable_fields/3","doc":"- The filtered query parameters.","ref":"FatEcto.Dynamics.FatBuildableHelper.html#filter_filterable_fields/3-returns"},{"type":"function","title":"FatEcto.Dynamics.FatBuildableHelper.remove_ignoreable_fields/2","doc":"Removes fields with ignoreable values from the query parameters.","ref":"FatEcto.Dynamics.FatBuildableHelper.html#remove_ignoreable_fields/2"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatBuildableHelper.remove_ignoreable_fields/2","doc":"- `where_params`: The query parameters (e.g., `%{\"field\" => %{\"$EQUAL\" => \"value\"}}`).\n  - `ignoreable_fields_values`: A map of fields and their ignoreable values (e.g., `%{\"email\" => [\"%%\", \"\", [], nil]}`).","ref":"FatEcto.Dynamics.FatBuildableHelper.html#remove_ignoreable_fields/2-parameters"},{"type":"function","title":"Returns - FatEcto.Dynamics.FatBuildableHelper.remove_ignoreable_fields/2","doc":"- The filtered query parameters.","ref":"FatEcto.Dynamics.FatBuildableHelper.html#remove_ignoreable_fields/2-returns"},{"type":"module","title":"FatEcto.Dynamics.FatDynamics","doc":"Provides functions to build dynamic `where` conditions for Ecto queries.\n\nThis module is designed to construct complex `where` clauses dynamically using Ecto's `dynamic/2` macro.\nIt supports a variety of conditions such as equality, inequality, comparison operators (`>`, `<`, `>=`, `<=`),\n`IN` clauses, `LIKE`/`ILIKE` for string matching, and JSONB operations (`@>`, `&&`).","ref":"FatEcto.Dynamics.FatDynamics.html"},{"type":"module","title":"Key Features: - FatEcto.Dynamics.FatDynamics","doc":"- **Dynamic Conditions**: Builds `where` clauses dynamically based on field names, values, and logic types (`:and`/`:or`).\n- **Binding Support**: Supports query bindings for joins or nested queries via the `:binding` option.\n- **JSONB Operations**: Provides functions to work with JSONB fields, such as checking containment or overlap.\n- **String Matching**: Supports `LIKE` and `ILIKE` for substring matching, including case-insensitive searches.\n- **Range Queries**: Allows building conditions for ranges (`BETWEEN`, `IN`, etc.).","ref":"FatEcto.Dynamics.FatDynamics.html#module-key-features"},{"type":"module","title":"Usage: - FatEcto.Dynamics.FatDynamics","doc":"Each function in this module constructs a dynamic expression that can be combined with other dynamics or used directly\nin an Ecto query. The functions accept the following common parameters:\n- `key`: The field name (as a string or atom).\n- `value`: The value to compare against (can be a single value, list, or map depending on the function).\n- `opts`: Options to control the behavior, such `:binding` (`:last` for joins).","ref":"FatEcto.Dynamics.FatDynamics.html#module-usage"},{"type":"module","title":"Example: - FatEcto.Dynamics.FatDynamics","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.field_is_nil_dynamic(:location)\n  iex> inspect(result)\n  \"dynamic([c], is_nil(c.location))\"\n\nThis module is typically used internally by `FatEcto` to construct queries based on user-provided filters.","ref":"FatEcto.Dynamics.FatDynamics.html#module-example"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.array_ilike_dynamic/3","doc":"Builds a dynamic query where any element in an array field matches a substring (case-insensitive).","ref":"FatEcto.Dynamics.FatDynamics.html#array_ilike_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.array_ilike_dynamic/3","doc":"- `key`       - The field name.\n  - `value`     - The substring to match.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#array_ilike_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.array_ilike_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.array_ilike_dynamic(:tags, \"%elixir%\")\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"EXISTS (SELECT 1 FROM UNNEST(?) as value WHERE value ILIKE ?)\\\", q.tags, ^\\\"%elixir%\\\"))\"","ref":"FatEcto.Dynamics.FatDynamics.html#array_ilike_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.between_dynamic/3","doc":"Builds a dynamic query where a field's value is between two provided values.","ref":"FatEcto.Dynamics.FatDynamics.html#between_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.between_dynamic/3","doc":"- `key`       - The field name.\n  - `values`    - A list of two values representing the range.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#between_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.between_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.between_dynamic(:experience_years, [2, 5])\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years > ^2 and q.experience_years < ^5)\"","ref":"FatEcto.Dynamics.FatDynamics.html#between_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.between_equal_dynamic/3","doc":"Builds a dynamic query where a field's value is between or equal to two provided values.","ref":"FatEcto.Dynamics.FatDynamics.html#between_equal_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.between_equal_dynamic/3","doc":"- `key`       - The field name.\n  - `values`    - A list of two values representing the range.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#between_equal_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.between_equal_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.between_equal_dynamic(:experience_years, [2, 5])\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years >= ^2 and q.experience_years <= ^5)\"","ref":"FatEcto.Dynamics.FatDynamics.html#between_equal_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.cast_to_date_eq_dynamic/3","doc":"Builds a dynamic query where a field casted to date is equal to a value.","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_eq_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.cast_to_date_eq_dynamic/3","doc":"- `key`       - The field name to cast into date.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_eq_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.cast_to_date_eq_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.cast_to_date_eq_dynamic(:end_date, ~D[2025-02-08])\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"?::date\\\", q.end_date == ^~D[2025-02-08]))\"","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_eq_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.cast_to_date_gt_dynamic/3","doc":"Builds a dynamic query where a field casted to date is greater than a value.","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_gt_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.cast_to_date_gt_dynamic/3","doc":"- `key`       - The field name to cast into date.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_gt_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.cast_to_date_gt_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.cast_to_date_gt_dynamic(:end_date, ~D[2025-02-08])\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"?::date\\\", q.end_date > ^~D[2025-02-08]))\"","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_gt_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.cast_to_date_gte_dynamic/3","doc":"Builds a dynamic query where a field casted to date is great than or equal to a value.","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_gte_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.cast_to_date_gte_dynamic/3","doc":"- `key`       - The field name to cast into date.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_gte_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.cast_to_date_gte_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.cast_to_date_gte_dynamic(:end_date, ~D[2025-02-08])\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"?::date\\\", q.end_date >= ^~D[2025-02-08]))\"","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_gte_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.cast_to_date_lt_dynamic/3","doc":"Builds a dynamic query where a field casted to date is less than a value.","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_lt_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.cast_to_date_lt_dynamic/3","doc":"- `key`       - The field name to cast into date.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_lt_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.cast_to_date_lt_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.cast_to_date_lt_dynamic(:end_date, ~D[2025-02-08])\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"?::date\\\", q.end_date < ^~D[2025-02-08]))\"","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_lt_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.cast_to_date_lte_dynamic/3","doc":"Builds a dynamic query where a field casted to date is less than or equal to a value.","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_lte_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.cast_to_date_lte_dynamic/3","doc":"- `key`       - The field name to cast into date.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_lte_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.cast_to_date_lte_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.cast_to_date_lte_dynamic(:end_date, ~D[2025-02-08])\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"?::date\\\", q.end_date <= ^~D[2025-02-08]))\"","ref":"FatEcto.Dynamics.FatDynamics.html#cast_to_date_lte_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.contains_any_dynamic/3","doc":"Builds a dynamic query where a JSONB field contains any of the provided values.","ref":"FatEcto.Dynamics.FatDynamics.html#contains_any_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.contains_any_dynamic/3","doc":"- `key`       - The JSONB field name.\n  - `values`    - The values to check for overlap.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#contains_any_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.contains_any_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.contains_any_dynamic(:tags, [\"elixir\", \"erlang\"])\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"? && ?\\\", q.tags, ^[\\\"elixir\\\", \\\"erlang\\\"]))\"","ref":"FatEcto.Dynamics.FatDynamics.html#contains_any_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.contains_dynamic/3","doc":"Builds a dynamic query where a JSONB field contains a specific value.","ref":"FatEcto.Dynamics.FatDynamics.html#contains_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.contains_dynamic/3","doc":"- `key`       - The JSONB field name.\n  - `values`    - The value(s) to check for containment.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#contains_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.contains_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.contains_dynamic(:metadata, %{\"role\" => \"admin\"})\n    iex> inspect(result)\n    \"dynamic([q], fragment(\\\"? @> ?\\\", q.metadata, ^%{\\\"role\\\" => \\\"admin\\\"}))\"","ref":"FatEcto.Dynamics.FatDynamics.html#contains_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.eq_dynamic/3","doc":"Builds a dynamic query where a field is equal to a value.","ref":"FatEcto.Dynamics.FatDynamics.html#eq_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.eq_dynamic/3","doc":"- `key`       - The field name.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#eq_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.eq_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.eq_dynamic(:experience_years, 2)\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years == ^2)\"","ref":"FatEcto.Dynamics.FatDynamics.html#eq_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.field_is_nil_dynamic/2","doc":"Builds a dynamic query where field is nil.\n\nParameters\n- `key`       - Field name.\n- `opts`      - Options for binding.\nExamples\n  iex> result = Elixir.FatEcto.Dynamics.FatDynamics.field_is_nil_dynamic(:location)\n  iex> inspect(result)\n  \"dynamic([c], is_nil(c.location))\"","ref":"FatEcto.Dynamics.FatDynamics.html#field_is_nil_dynamic/2"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.gt_dynamic/3","doc":"Builds dynamic condition for field greater than value.\n\nParameters\n - `key`       - Field name.\n - `value`     - Comparison value or field reference.\n - `opts`      - Options for binding\nExamples\n  iex> result = Elixir.FatEcto.Dynamics.FatDynamics.gt_dynamic(:experience_years, 2, [binding: :last])\n  iex> inspect(result)\n  \"dynamic([_, ..., c], c.experience_years > ^2)\"","ref":"FatEcto.Dynamics.FatDynamics.html#gt_dynamic/3"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.gte_dynamic/3","doc":"Builds dynamic condition for field greater than or equal to value.\n\nParameters\n - `key`       - Field name.\n - `value`     - Comparison value or field reference.\n - `opts`      - Options for binding\nExamples\n  iex> result = Elixir.FatEcto.Dynamics.FatDynamics.gte_dynamic(:experience_years, 2, [binding: :last])\n  iex> inspect(result)\n  \"dynamic([_, ..., c], c.experience_years >= ^2)\"","ref":"FatEcto.Dynamics.FatDynamics.html#gte_dynamic/3"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.ilike_dynamic/3","doc":"Builds a dynamic query where a field matches a substring (case-insensitive).","ref":"FatEcto.Dynamics.FatDynamics.html#ilike_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.ilike_dynamic/3","doc":"- `key`       - The field name.\n  - `value`     - The substring to match.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#ilike_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.ilike_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.ilike_dynamic(:name, \"%john%\")\n    iex> inspect(result)\n    \"dynamic([q], ilike(fragment(\\\"(?)::TEXT\\\", q.name), ^\\\"%john%\\\"))\"","ref":"FatEcto.Dynamics.FatDynamics.html#ilike_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.in_dynamic/3","doc":"Builds a dynamic query where a field's value is in a provided list.","ref":"FatEcto.Dynamics.FatDynamics.html#in_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.in_dynamic/3","doc":"- `key`       - The field name.\n  - `values`    - A list of values to match against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#in_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.in_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.in_dynamic(:experience_years, [2, 5])\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years in ^[2, 5])\"","ref":"FatEcto.Dynamics.FatDynamics.html#in_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.like_dynamic/3","doc":"Builds a dynamic query where a field matches a substring (case-sensitive).","ref":"FatEcto.Dynamics.FatDynamics.html#like_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.like_dynamic/3","doc":"- `key`       - The field name.\n  - `value`     - The substring to match.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#like_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.like_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.like_dynamic(:name, \"%John%\")\n    iex> inspect(result)\n    \"dynamic([q], like(fragment(\\\"(?)::TEXT\\\", q.name), ^\\\"%John%\\\"))\"","ref":"FatEcto.Dynamics.FatDynamics.html#like_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.lt_dynamic/3","doc":"Builds a dynamic query where a field is less than a given value.","ref":"FatEcto.Dynamics.FatDynamics.html#lt_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.lt_dynamic/3","doc":"- `key`       - The field name.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#lt_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.lt_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.lt_dynamic(:experience_years, 2, [binding: :last])\n    iex> inspect(result)\n    \"dynamic([_, ..., c], c.experience_years < ^2)\"","ref":"FatEcto.Dynamics.FatDynamics.html#lt_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.lte_dynamic/3","doc":"Builds dynamic condition for field less than or equal to value.\n\nParameters\n - `key`       - Field name.\n - `value`     - Comparison value or field reference.\n - `opts`      - Options for binding\nExamples\n  iex> result = Elixir.FatEcto.Dynamics.FatDynamics.lte_dynamic(:experience_years, 2)\n  iex> inspect(result)\n  \"dynamic([q], q.experience_years <= ^2)\"","ref":"FatEcto.Dynamics.FatDynamics.html#lte_dynamic/3"},{"type":"function","title":"FatEcto.Dynamics.FatDynamics.not_eq_dynamic/3","doc":"Builds a dynamic query where a field is not equal to a value.","ref":"FatEcto.Dynamics.FatDynamics.html#not_eq_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatDynamics.not_eq_dynamic/3","doc":"- `key`       - The field name.\n  - `value`     - The value to compare against.\n  - `opts`      - Options for binding and logic type (:and/:or).","ref":"FatEcto.Dynamics.FatDynamics.html#not_eq_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamics.not_eq_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatDynamics.not_eq_dynamic(:experience_years, 2)\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years != ^2)\"","ref":"FatEcto.Dynamics.FatDynamics.html#not_eq_dynamic/3-examples"},{"type":"module","title":"FatEcto.Dynamics.FatDynamicsBuilder","doc":"This module builds Ecto dynamic queries from a JSON-like structure.\nIt uses the `FatEcto.Dynamics.FatOperatorHelper` module to apply operators and construct the query.","ref":"FatEcto.Dynamics.FatDynamicsBuilder.html"},{"type":"function","title":"FatEcto.Dynamics.FatDynamicsBuilder.build/3","doc":"Builds an Ecto dynamic query from a JSON-like structure.","ref":"FatEcto.Dynamics.FatDynamicsBuilder.html#build/3"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatDynamicsBuilder.build/3","doc":"iex> import Ecto.Query\n    ...> query = FatEcto.Dynamics.FatDynamicsBuilder.build(%{\n    ...>   \"$OR\" => [\n    ...>     %{\"name\" => \"John\"},\n    ...>     %{\"phone\" => nil},\n    ...>     %{\"age\" => %{\"$GT\" => 30}}\n    ...>   ]\n    ...> })\n    iex> inspect(query)\n    \"dynamic([q], q.name == ^\\\"John\\\" or is_nil(q.phone) or q.age > ^30)\"","ref":"FatEcto.Dynamics.FatDynamicsBuilder.html#build/3-examples"},{"type":"module","title":"FatEcto.Dynamics.FatNotDynamics","doc":"Builds a `where query` using dynamics and not condition.","ref":"FatEcto.Dynamics.FatNotDynamics.html"},{"type":"module","title":"Parameters - FatEcto.Dynamics.FatNotDynamics","doc":"- `queryable`   - Ecto Queryable that represents your schema name, table name or query.\n  - `query_opts`  - Where query options as a map.","ref":"FatEcto.Dynamics.FatNotDynamics.html#module-parameters"},{"type":"module","title":"Examples - FatEcto.Dynamics.FatNotDynamics","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_field_is_nil_dynamic(:location)\n  iex> inspect(result)\n  \"dynamic([c], not is_nil(c.location))\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#module-examples"},{"type":"module","title":"Options - FatEcto.Dynamics.FatNotDynamics","doc":"- `$select` - Select the fields from `hospital` and `rooms`.\n  - `$where`  - Added the where attribute in the query.","ref":"FatEcto.Dynamics.FatNotDynamics.html#module-options"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.eq_dynamic/3","doc":"Builds a dynamic query where field is equal to the value.","ref":"FatEcto.Dynamics.FatNotDynamics.html#eq_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.eq_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#eq_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.eq_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.eq_dynamic(:experience_years, 2)\n     iex> inspect(result)\n     \"dynamic([q], q.experience_years == ^2)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#eq_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_between_dynamic/3","doc":"Builds a dynamic query where value is not between the provided attributes.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_between_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_between_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Values of the field as a list.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_between_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_between_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_between_dynamic(:experience_years, [2, 5])\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years < ^2 or q.experience_years > ^5)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_between_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_between_equal_dynamic/3","doc":"Builds a dynamic query where value is not equal and between the provided attributes.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_between_equal_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_between_equal_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Values of the field as a list.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_between_equal_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_between_equal_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_between_equal_dynamic(:experience_years, [2, 5])\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years <= ^2 or q.experience_years >= ^5)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_between_equal_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_contains_any_dynamic/3","doc":"Builds a dynamic query when value of jsonb not matches with any list attribute.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_contains_any_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_contains_any_dynamic/3","doc":"- `key`       - Field name.\n   - `values`    - values of jsonb field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_contains_any_dynamic/3-parameters"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_contains_dynamic/3","doc":"Builds a dynamic query when value of jsonb field is not in the list.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_contains_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_contains_dynamic/3","doc":"- `key`       - Field name.\n   - `values`    - values of jsonb field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_contains_dynamic/3-parameters"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_eq_dynamic/3","doc":"Builds a dynamic query where field is not equal to the value.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_eq_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_eq_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_eq_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_eq_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_eq_dynamic(:experience_years, 2)\n     iex> inspect(result)\n     \"dynamic([q], q.experience_years != ^2)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_eq_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_field_is_nil_dynamic/2","doc":"Builds a dynamic query where field is not nil.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_field_is_nil_dynamic/2"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_field_is_nil_dynamic/2","doc":"- `key`       - Field name .\n  - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_field_is_nil_dynamic/2-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_field_is_nil_dynamic/2","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_field_is_nil_dynamic(:location)\n    iex> inspect(result)\n    \"dynamic([c], not is_nil(c.location))\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_field_is_nil_dynamic/2-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_gt_dynamic/3","doc":"Builds a dynamic query where field is not greater than the value.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_gt_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_gt_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_gt_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_gt_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_gt_dynamic(:experience_years, 2, [binding: :last])\n    iex> inspect(result)\n    \"dynamic([_, ..., c], c.experience_years < ^2)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_gt_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_gte_dynamic/3","doc":"Builds a dynamic query where field is not greater than or equal to the value.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_gte_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_gte_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_gte_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_gte_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_gte_dynamic(:experience_years, 2, [binding: :last])\n    iex> inspect(result)\n    \"dynamic([_, ..., c], c.experience_years < ^2)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_gte_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_ilike_dynamic/3","doc":"Builds a dynamic query where field not matches the value substring.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_ilike_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_ilike_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_ilike_dynamic/3-parameters"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_in_dynamic/3","doc":"Builds a dynamic query where value is not in the the list attributes.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_in_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_in_dynamic/3","doc":"- `key`       - Field name.\n   - `values`    - Pass a list of values of the field that represent range.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_in_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_in_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_in_dynamic(:experience_years, [2, 5])\n    iex> inspect(result)\n    \"dynamic([q], q.experience_years not in ^[2, 5])\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_in_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_like_dynamic/3","doc":"Builds a dynamic query where field not matches the value substring.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_like_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_like_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_like_dynamic/3-parameters"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_lt_dynamic/3","doc":"Builds a dynamic query where field is not less than the value.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_lt_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_lt_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_lt_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_lt_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_lt_dynamic(:experience_years, 2, [binding: :last])\n    iex> inspect(result)\n    \"dynamic([_, ..., c], c.experience_years > ^2)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_lt_dynamic/3-examples"},{"type":"function","title":"FatEcto.Dynamics.FatNotDynamics.not_lte_dynamic/3","doc":"Builds a dynamic query where field is not less than or equal to the value.","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_lte_dynamic/3"},{"type":"function","title":"Parameters - FatEcto.Dynamics.FatNotDynamics.not_lte_dynamic/3","doc":"- `key`       - Field name.\n   - `value`     - Value of the field.\n   - `opts`      - Options related to query bindings","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_lte_dynamic/3-parameters"},{"type":"function","title":"Examples - FatEcto.Dynamics.FatNotDynamics.not_lte_dynamic/3","doc":"iex> result = Elixir.FatEcto.Dynamics.FatNotDynamics.not_lte_dynamic(:experience_years, 2, [binding: :last])\n    iex> inspect(result)\n    \"dynamic([c], c.experience_years > ^2)\"","ref":"FatEcto.Dynamics.FatNotDynamics.html#not_lte_dynamic/3-examples"},{"type":"module","title":"FatEcto.Dynamics.FatOperatorHelper","doc":"Provides helper functions to apply dynamic query operators for Ecto queries.\n\nThis module centralizes the logic for applying various query operators such as `$LIKE`, `$ILIKE`, `$LT`, `$GT`, etc.,\nas well as their negated counterparts (e.g., `$NOT_LIKE`, `$NOT_ILIKE`). It is designed to work with `FatDynamics` and\n`FatNotDynamics` modules to generate dynamic Ecto query conditions.","ref":"FatEcto.Dynamics.FatOperatorHelper.html"},{"type":"module","title":"Supported Operators - FatEcto.Dynamics.FatOperatorHelper","doc":"","ref":"FatEcto.Dynamics.FatOperatorHelper.html#module-supported-operators"},{"type":"module","title":"Positive Operators - FatEcto.Dynamics.FatOperatorHelper","doc":"- `$LIKE`: Matches a pattern in a string.\n- `$NOT_LIKE`: Excludes rows that match a pattern in a string.\n- `$ILIKE`: Case-insensitive match of a pattern in a string.\n- `$NOT_ILIKE`: Case-insensitive exclusion of rows that match a pattern in a string.\n- `$LT`: Less than.\n- `$LTE`: Less than or equal to.\n- `$GT`: Greater than.\n- `$GTE`: Greater than or equal to.\n- `$BETWEEN`: Matches values between a range (exclusive).\n- `$BETWEEN_EQUAL`: Matches values between a range (inclusive).\n- `$NOT_BETWEEN`: Excludes values between a range (exclusive).\n- `$NOT_BETWEEN_EQUAL`: Excludes values between a range (inclusive).\n- `$IN`: Matches values in a list.\n- `$NOT_IN`: Excludes values in a list.\n- `$EQUAL`: Matches a specific value.\n- `$NOT_EQUAL`: Excludes a specific value.","ref":"FatEcto.Dynamics.FatOperatorHelper.html#module-positive-operators"},{"type":"module","title":"Negated Operators (for `$NOT` conditions) - FatEcto.Dynamics.FatOperatorHelper","doc":"- `$LIKE`: Negates the `$LIKE` condition.\n- `$ILIKE`: Negates the `$ILIKE` condition.\n- `$LT`: Negates the `$LT` condition.\n- `$LTE`: Negates the `$LTE` condition.\n- `$GT`: Negates the `$GT` condition.\n- `$GTE`: Negates the `$GTE` condition.\n- `$EQUAL`: Negates the `$EQUAL` condition.","ref":"FatEcto.Dynamics.FatOperatorHelper.html#module-negated-operators-for-not-conditions"},{"type":"module","title":"Usage - FatEcto.Dynamics.FatOperatorHelper","doc":"This module is typically used internally by `FatEcto.FatQuery.FatWhere` and `FatEcto.FatQuery.WhereOr` to construct\ndynamic queries. It abstracts away the complexity of applying operators and ensures consistency across the codebase.","ref":"FatEcto.Dynamics.FatOperatorHelper.html#module-usage"},{"type":"module","title":"Example - FatEcto.Dynamics.FatOperatorHelper","doc":"# Applying a `$LIKE` operator\n    FatOperatorHelper.apply_operator(\"$LIKE\", :name, \"%John%\", [])\n\n    # Applying a negated `$EQUAL` operator\n    FatOperatorHelper.apply_not_condition(\"$EQUAL\", :age, 30, [])","ref":"FatEcto.Dynamics.FatOperatorHelper.html#module-example"},{"type":"function","title":"FatEcto.Dynamics.FatOperatorHelper.allowed_not_operators/0","doc":"","ref":"FatEcto.Dynamics.FatOperatorHelper.html#allowed_not_operators/0"},{"type":"function","title":"FatEcto.Dynamics.FatOperatorHelper.allowed_operators/0","doc":"","ref":"FatEcto.Dynamics.FatOperatorHelper.html#allowed_operators/0"},{"type":"function","title":"FatEcto.Dynamics.FatOperatorHelper.apply_nil_operator/3","doc":"","ref":"FatEcto.Dynamics.FatOperatorHelper.html#apply_nil_operator/3"},{"type":"function","title":"FatEcto.Dynamics.FatOperatorHelper.apply_not_condition/4","doc":"","ref":"FatEcto.Dynamics.FatOperatorHelper.html#apply_not_condition/4"},{"type":"function","title":"FatEcto.Dynamics.FatOperatorHelper.apply_operator/4","doc":"","ref":"FatEcto.Dynamics.FatOperatorHelper.html#apply_operator/4"},{"type":"module","title":"FatEcto.FatContext","doc":"Provides a set of utility functions to simplify common Ecto operations such as querying, creating, updating, and deleting records.\n\nThis module is designed to be used within a context module to provide a consistent and easy-to-use API for interacting with your database.","ref":"FatEcto.FatContext.html"},{"type":"module","title":"Usage - FatEcto.FatContext","doc":"defmodule FatEcto.FatAppContext do\n      use FatEcto.FatContext, repo: FatEcto.Repo\n\n      # Custom functions can be added here\n    end\n\nNow you can use the functions provided by `FatEcto.FatContext` within `FatEcto.FatAppContext`.","ref":"FatEcto.FatContext.html#module-usage"},{"type":"function","title":"FatEcto.FatContext.__after_compile__/2","doc":"Callback function that runs after the module is compiled.","ref":"FatEcto.FatContext.html#__after_compile__/2"},{"type":"module","title":"FatEcto.FatDataSanitizer","doc":"Provides functionality to sanitize records and convert data into views.\n\nThis module can be used to clean up records by removing sensitive data, masking specific fields,\nand customizing the output based on options.","ref":"FatEcto.FatDataSanitizer.html"},{"type":"module","title":"FatEcto.FatHelper","doc":"Provides utility functions for FatEcto, including handling pagination limits, skip values,\ndynamic binding, and preloading associations.","ref":"FatEcto.FatHelper.html"},{"type":"function","title":"FatEcto.FatHelper.fat_ecto_reserve_field?/1","doc":"Determines if a value is a reserved field in FatEcto.","ref":"FatEcto.FatHelper.html#fat_ecto_reserve_field?/1"},{"type":"function","title":"Parameters - FatEcto.FatHelper.fat_ecto_reserve_field?/1","doc":"- `value`: The value to check.","ref":"FatEcto.FatHelper.html#fat_ecto_reserve_field?/1-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.fat_ecto_reserve_field?/1","doc":"iex> FatEcto.FatHelper.fat_ecto_reserve_field?(\"$INCLUDE\")\n    true","ref":"FatEcto.FatHelper.html#fat_ecto_reserve_field?/1-examples"},{"type":"function","title":"FatEcto.FatHelper.get_limit_bounds/1","doc":"Returns the maximum and default limit values based on the provided options.","ref":"FatEcto.FatHelper.html#get_limit_bounds/1"},{"type":"function","title":"Parameters - FatEcto.FatHelper.get_limit_bounds/1","doc":"- `options`: A keyword list or map containing `max_limit` and `default_limit`.","ref":"FatEcto.FatHelper.html#get_limit_bounds/1-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.get_limit_bounds/1","doc":"iex> FatEcto.FatHelper.get_limit_bounds(max_limit: 50, default_limit: 10)\n    {50, 10}","ref":"FatEcto.FatHelper.html#get_limit_bounds/1-examples"},{"type":"function","title":"FatEcto.FatHelper.get_limit_value/2","doc":"Extracts and validates the limit value from the given parameters.","ref":"FatEcto.FatHelper.html#get_limit_value/2"},{"type":"function","title":"Parameters - FatEcto.FatHelper.get_limit_value/2","doc":"- `params`: A keyword list containing the `:limit` value.\n- `options`: A keyword list or map containing `max_limit` and `default_limit`.","ref":"FatEcto.FatHelper.html#get_limit_value/2-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.get_limit_value/2","doc":"iex> FatEcto.FatHelper.get_limit_value([limit: 15], max_limit: 50, default_limit: 10)\n    {15, []}","ref":"FatEcto.FatHelper.html#get_limit_value/2-examples"},{"type":"function","title":"FatEcto.FatHelper.get_primary_keys/1","doc":"Retrieves the primary keys for a given query.","ref":"FatEcto.FatHelper.html#get_primary_keys/1"},{"type":"function","title":"Parameters - FatEcto.FatHelper.get_primary_keys/1","doc":"- `query`: The Ecto query.","ref":"FatEcto.FatHelper.html#get_primary_keys/1-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.get_primary_keys/1","doc":"iex> FatEcto.FatHelper.get_primary_keys(from(u in User))\n    [:id]","ref":"FatEcto.FatHelper.html#get_primary_keys/1-examples"},{"type":"function","title":"FatEcto.FatHelper.get_skip_value/1","doc":"Extracts and validates the skip value from the given parameters.","ref":"FatEcto.FatHelper.html#get_skip_value/1"},{"type":"function","title":"Parameters - FatEcto.FatHelper.get_skip_value/1","doc":"- `params`: A keyword list containing the `:skip` value.","ref":"FatEcto.FatHelper.html#get_skip_value/1-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.get_skip_value/1","doc":"iex> FatEcto.FatHelper.get_skip_value(skip: 20)\n    {20, []}","ref":"FatEcto.FatHelper.html#get_skip_value/1-examples"},{"type":"function","title":"FatEcto.FatHelper.implements_behaviour?/2","doc":"Checks if a module implements the given behaviour by inspecting its `__info__/1` metadata.","ref":"FatEcto.FatHelper.html#implements_behaviour?/2"},{"type":"function","title":"Parameters - FatEcto.FatHelper.implements_behaviour?/2","doc":"- `module`: The module to check.\n- `behaviour`: The behaviour to validate against.","ref":"FatEcto.FatHelper.html#implements_behaviour?/2-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.implements_behaviour?/2","doc":"iex> implements_behaviour?(MyApp.Repo, Ecto.Repo)\n    true\n\n    iex> implements_behaviour?(NotARepo, Ecto.Repo)\n    false","ref":"FatEcto.FatHelper.html#implements_behaviour?/2-examples"},{"type":"function","title":"FatEcto.FatHelper.string_to_atom/1","doc":"Converts a string to an atom.","ref":"FatEcto.FatHelper.html#string_to_atom/1"},{"type":"function","title":"Parameters - FatEcto.FatHelper.string_to_atom/1","doc":"- `str`: The string to convert.","ref":"FatEcto.FatHelper.html#string_to_atom/1-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.string_to_atom/1","doc":"iex> FatEcto.FatHelper.string_to_atom(\"example\")\n    :example","ref":"FatEcto.FatHelper.html#string_to_atom/1-examples"},{"type":"function","title":"FatEcto.FatHelper.string_to_existing_atom/1","doc":"Converts a string to an existing atom.","ref":"FatEcto.FatHelper.html#string_to_existing_atom/1"},{"type":"function","title":"Parameters - FatEcto.FatHelper.string_to_existing_atom/1","doc":"- `str`: The string to convert.","ref":"FatEcto.FatHelper.html#string_to_existing_atom/1-parameters"},{"type":"function","title":"Examples - FatEcto.FatHelper.string_to_existing_atom/1","doc":"iex> FatEcto.FatHelper.string_to_existing_atom(\"example\")\n    :example","ref":"FatEcto.FatHelper.html#string_to_existing_atom/1-examples"},{"type":"module","title":"FatEcto.FatOrderBy","doc":"Builds query with `asc` or `desc` order.\n\nThis module provides functionality to dynamically add `order_by` clauses to Ecto queries\nbased on the provided parameters. It supports various order formats, including `$ASC`,\n`$DESC`, and nulls handling (`$ASC_NULLS_FIRST`, `$DESC_NULLS_LAST`, etc.).","ref":"FatEcto.FatOrderBy.html"},{"type":"function","title":"FatEcto.FatOrderBy.build_order_by/4","doc":"Orders the results based on the `order_by` clause in the params.","ref":"FatEcto.FatOrderBy.html#build_order_by/4"},{"type":"function","title":"Parameters - FatEcto.FatOrderBy.build_order_by/4","doc":"- `queryable`: Ecto Queryable that represents your schema name, table name, or query.\n  - `order_by_params`: A map of fields and their order formats (e.g., `%{\"field\" => \"$ASC\"}`).\n  - `opts`: Options related to query bindings.\n  - `build_options`: Options related to the OTP app (unused in this function).","ref":"FatEcto.FatOrderBy.html#build_order_by/4-parameters"},{"type":"function","title":"Examples - FatEcto.FatOrderBy.build_order_by/4","doc":"iex> query_opts = %{\"$ORder\" => %{\"id\" => \"$ASC\"}}\n    iex> FatEcto.FatOrderBy.build_order_by(FatEcto.FatHospital, query_opts[\"$ORder\"], [], [])\n    #Ecto.Query","ref":"FatEcto.FatOrderBy.html#build_order_by/4-examples"},{"type":"behaviour","title":"FatEcto.FatPaginator","doc":"Provides pagination functionality for Ecto queries.\n\nThis module can be used to paginate query results by specifying `limit` and `skip` parameters.\nIt also supports counting the total number of records for pagination metadata.","ref":"FatEcto.FatPaginator.html"},{"type":"behaviour","title":"Usage - FatEcto.FatPaginator","doc":"defmodule Fat.MyContext do\n      use FatEcto.FatPaginator, repo: Fat.Repo, default_limit: 10, max_limit: 100\n\n      # Custom functions can be added here\n    end\n\nNow you can use the `paginate/2`, `paginator/3`, and `paginate_get_records/3` functions within `Fat.MyContext`.","ref":"FatEcto.FatPaginator.html#module-usage"},{"type":"function","title":"FatEcto.FatPaginator.__after_compile__/2","doc":"Callback function that runs after the module is compiled.","ref":"FatEcto.FatPaginator.html#__after_compile__/2"},{"type":"callback","title":"FatEcto.FatPaginator.aggregate/1","doc":"","ref":"FatEcto.FatPaginator.html#c:aggregate/1"},{"type":"callback","title":"FatEcto.FatPaginator.count_query/1","doc":"","ref":"FatEcto.FatPaginator.html#c:count_query/1"},{"type":"callback","title":"FatEcto.FatPaginator.data_query/3","doc":"","ref":"FatEcto.FatPaginator.html#c:data_query/3"},{"type":"behaviour","title":"FatEcto.FatSortable","doc":"Provides functionality to sort Ecto queries based on user-defined rules.\n\nThis module allows sorting queries using predefined default fields (handled by `FatOrderBy`)\nand custom fields (handled by a fallback function).","ref":"FatEcto.FatSortable.html"},{"type":"behaviour","title":"Usage - FatEcto.FatSortable","doc":"defmodule Fat.SortQuery do\n      <!-- needed if you are writing queries in override_sortable -->\n      import Ecto.Query\n      use FatEcto.FatSortable,\n        sortable_fields: %{\"id\" => \"$ASC\", \"name\" => [\"$ASC\", \"$DESC\"]},\n        overrideable_fields: [\"custom_field\"]\n\n      @impl true\n      def override_sortable(query, field, operator) do\n        case {field, operator} do\n          {\"custom_field\", \"$ASC\"} ->\n            from(q in query, order_by: [asc: fragment(\"?::JSONB->>'custom_field'\", q)])\n          _ ->\n            query\n        end\n      end\n    end","ref":"FatEcto.FatSortable.html#module-usage"},{"type":"behaviour","title":"Example - FatEcto.FatSortable","doc":"query = from(u in User)\n    sort_params = %{\"id\" => \"$ASC\", \"name\" => \"$DESC\", \"custom_field\" => \"$ASC\"}\n    Fat.SortQuery.build(query, sort_params)\n\nThis will sort the query by `id` in ascending order, `name` in descending order, and apply custom sorting for `custom_field`.","ref":"FatEcto.FatSortable.html#module-example"},{"type":"callback","title":"FatEcto.FatSortable.override_sortable/3","doc":"Callback for handling custom sorting logic.\n\nThis function is called for fields defined in `overrideable_fields`. The default behavior is to return the query,\nbut it can be overridden by the using module.","ref":"FatEcto.FatSortable.html#c:override_sortable/3"},{"type":"module","title":"FatEcto.FatSortableHelper","doc":"Provides helper functions for sorting query parameters.","ref":"FatEcto.FatSortableHelper.html"},{"type":"function","title":"FatEcto.FatSortableHelper.filter_sortable_fields/2","doc":"Filters sortable fields based on the provided `sortable_fields` map.","ref":"FatEcto.FatSortableHelper.html#filter_sortable_fields/2"},{"type":"function","title":"Parameters - FatEcto.FatSortableHelper.filter_sortable_fields/2","doc":"- `filtered_params`: A map of fields and their sorting operators.\n  - `sortable_fields`: A map of allowed fields and their corresponding operators.","ref":"FatEcto.FatSortableHelper.html#filter_sortable_fields/2-parameters"},{"type":"function","title":"Returns - FatEcto.FatSortableHelper.filter_sortable_fields/2","doc":"- A filtered map containing only the fields and operators that are allowed.","ref":"FatEcto.FatSortableHelper.html#filter_sortable_fields/2-returns"},{"type":"behaviour","title":"FatEcto.Query.FatBuildable","doc":"Builds queries after filtering fields based on user-provided filterable and overrideable fields.\n\nThis module provides functionality to filter Ecto queries using predefined filterable fields (handled by `FatWhere`)\nand overrideable fields (handled by a fallback function).","ref":"FatEcto.Query.FatBuildable.html"},{"type":"behaviour","title":"Options - FatEcto.Query.FatBuildable","doc":"- `filterable_fields`: A map of fields and their allowed operators. Example:\n    %{\n      \"id\" => [\"$eq\", \"$neq\"],\n      \"name\" => [\"$ilike\"]\n    }\n- `overrideable_fields`: A list of fields that can be overridden. Example:\n    [\"name\", \"phone\"]\n- `ignoreable_fields_values`: A map of fields and their ignoreable values. Example:\n    %{\n      \"name\" => [\"%%\", \"\", [], nil],\n      \"phone\" => [\"%%\", \"\", [], nil]\n    }","ref":"FatEcto.Query.FatBuildable.html#module-options"},{"type":"behaviour","title":"Example Usage - FatEcto.Query.FatBuildable","doc":"defmodule MyApp.HospitalFilter do\n      use FatEcto.Query.FatBuildable,\n        filterable_fields: %{\n          \"id\" => [\"$eq\", \"$neq\"]\n        },\n        overrideable_fields: [\"name\", \"phone\"],\n        ignoreable_fields_values: %{\n          \"name\" => [\"%%\", \"\", [], nil],\n          \"phone\" => [\"%%\", \"\", [], nil]\n        }\n\n      import Ecto.Query\n\n      def override_whereable(query, \"name\", \"$ilike\", value) do\n        where(query, [r], ilike(fragment(\"(?)::TEXT\", r.name), ^value))\n      end\n\n      def override_whereable(query, \"phone\", \"$ilike\", value) do\n        where(query, [r], ilike(fragment(\"(?)::TEXT\", r.phone), ^value))\n      end\n\n      def override_whereable(query, _, _, _) do\n        query\n      end\n    end","ref":"FatEcto.Query.FatBuildable.html#module-example-usage"},{"type":"callback","title":"FatEcto.Query.FatBuildable.after_whereable/1","doc":"Callback for performing custom processing on the final query.\n\nThis function is called at the end of the `build/2` function. The default behavior is to return the query,\nbut it can be overridden by the using module.","ref":"FatEcto.Query.FatBuildable.html#c:after_whereable/1"},{"type":"callback","title":"FatEcto.Query.FatBuildable.override_whereable/4","doc":"Callback for handling custom filtering logic for overrideable fields.\n\nThis function acts as a fallback for overrideable fields. The default behavior is to return the query,\nbut it can be overridden by the using module.","ref":"FatEcto.Query.FatBuildable.html#c:override_whereable/4"},{"type":"module","title":"FatEcto.Query.FatQueryBuilder","doc":"This module builds Ecto queries from a JSON-like structure.\nIt uses the `FatEcto.Dynamics.FatOperatorHelper` module to apply operators and construct the query.","ref":"FatEcto.Query.FatQueryBuilder.html"},{"type":"function","title":"FatEcto.Query.FatQueryBuilder.build/3","doc":"Builds an Ecto query from a JSON-like structure.","ref":"FatEcto.Query.FatQueryBuilder.html#build/3"},{"type":"function","title":"Examples - FatEcto.Query.FatQueryBuilder.build/3","doc":"iex> import Ecto.Query\n    ...> query = FatEcto.Query.FatQueryBuilder.build(User, %{\n    ...>   \"$OR\" => [\n    ...>     %{\"name\" => \"John\"},\n    ...>     %{\"phone\" => nil},\n    ...>     %{\"age\" => %{\"$GT\" => 30}}\n    ...>   ]\n    ...> })\n    iex> inspect(query)\n    \"#Ecto.Query  ^30 or (is_nil(u.phone) or u.name == ^\"John\">\"","ref":"FatEcto.Query.FatQueryBuilder.html#build/3-examples"},{"type":"module","title":"FatEcto.Utils.Changeset","doc":"Provides utility functions for validating Ecto changesets.\n\nThis module includes functions for validating XOR conditions, requiring specific fields,\nand comparing datetime fields.","ref":"FatEcto.Utils.Changeset.html"},{"type":"function","title":"FatEcto.Utils.Changeset.add_custom_error/3","doc":"Adds a custom error to the changeset.","ref":"FatEcto.Utils.Changeset.html#add_custom_error/3"},{"type":"function","title":"Parameters - FatEcto.Utils.Changeset.add_custom_error/3","doc":"- `changeset`: The Ecto changeset.\n- `error_message_title`: The field or title for the error.\n- `error_message`: The error message (default: \"is invalid\").","ref":"FatEcto.Utils.Changeset.html#add_custom_error/3-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Changeset.add_custom_error/3","doc":"iex> import Ecto.Changeset\n    iex> changeset = cast(%FatEcto.FatUser{}, %{}, [])\n    iex> FatEcto.Utils.Changeset.add_custom_error(changeset, :field, \"custom error\")\n    #Ecto.Changeset<...>","ref":"FatEcto.Utils.Changeset.html#add_custom_error/3-examples"},{"type":"function","title":"FatEcto.Utils.Changeset.require_field_if_present/2","doc":"Makes a field required if another field is present in the changeset.","ref":"FatEcto.Utils.Changeset.html#require_field_if_present/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Changeset.require_field_if_present/2","doc":"- `changeset`: The Ecto changeset.\n- `if_change_key`: The key to check for presence.\n- `require_key`: The key to make required.","ref":"FatEcto.Utils.Changeset.html#require_field_if_present/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Changeset.require_field_if_present/2","doc":"iex> import Ecto.Changeset\n    iex> changeset = cast(%FatEcto.FatUser{}, %{field1: \"value1\"}, [:field1, :field2])\n    iex> FatEcto.Utils.Changeset.require_field_if_present(changeset, if_change_key: :field1, require_key: :field2)\n    #Ecto.Changeset<...>","ref":"FatEcto.Utils.Changeset.html#require_field_if_present/2-examples"},{"type":"function","title":"FatEcto.Utils.Changeset.validate_at_least_one_field/4","doc":"Validates that at least one of the specified OR keys is present in the changeset.","ref":"FatEcto.Utils.Changeset.html#validate_at_least_one_field/4"},{"type":"function","title":"Parameters - FatEcto.Utils.Changeset.validate_at_least_one_field/4","doc":"- `changeset`: The Ecto changeset.\n- `record`: The original record (currently unused).\n- `or_keys`: A list of keys to validate.\n- `options`: Additional options (currently unused).","ref":"FatEcto.Utils.Changeset.html#validate_at_least_one_field/4-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Changeset.validate_at_least_one_field/4","doc":"iex> import Ecto.Changeset\n    iex> changeset = cast(%FatEcto.FatUser{}, %{}, [:field1, :field2])\n    iex> FatEcto.Utils.Changeset.validate_at_least_one_field(changeset, %FatEcto.FatUser{}, [:field1, :field2])\n    #Ecto.Changeset<...>","ref":"FatEcto.Utils.Changeset.html#validate_at_least_one_field/4-examples"},{"type":"function","title":"FatEcto.Utils.Changeset.validate_only_one_field/4","doc":"Validates that only one of the specified keys is present in the changeset.","ref":"FatEcto.Utils.Changeset.html#validate_only_one_field/4"},{"type":"function","title":"Parameters - FatEcto.Utils.Changeset.validate_only_one_field/4","doc":"- `changeset`: The Ecto changeset.\n- `record`: The original record (currently unused).\n- `single_keys`: A list of keys to validate.\n- `options`: Additional options (currently unused).","ref":"FatEcto.Utils.Changeset.html#validate_only_one_field/4-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Changeset.validate_only_one_field/4","doc":"iex> import Ecto.Changeset\n    iex> changeset = cast(%FatEcto.FatUser{}, %{field1: \"value1\"}, [:field1, :field2])\n    iex> FatEcto.Utils.Changeset.validate_only_one_field(changeset, %FatEcto.FatUser{}, [:field1, :field2])\n    #Ecto.Changeset<...>","ref":"FatEcto.Utils.Changeset.html#validate_only_one_field/4-examples"},{"type":"function","title":"FatEcto.Utils.Changeset.validate_start_before_end/4","doc":"Validates that a start date/time is before an end date/time.","ref":"FatEcto.Utils.Changeset.html#validate_start_before_end/4"},{"type":"function","title":"Parameters - FatEcto.Utils.Changeset.validate_start_before_end/4","doc":"- `changeset`: The Ecto changeset.\n- `start_date_key`: The key for the start date/time.\n- `end_date_key`: The key for the end date/time.\n- `options`: Options to customize the error message or specify comparison type (`:time` or `:datetime`).","ref":"FatEcto.Utils.Changeset.html#validate_start_before_end/4-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Changeset.validate_start_before_end/4","doc":"iex> import Ecto.Changeset\n    iex> changeset = cast(%FatEcto.FatUser{}, %{start_time: ~T[10:00:00], end_time: ~T[09:00:00]}, [:start_time, :end_time])\n    iex> FatEcto.Utils.Changeset.validate_start_before_end(changeset, :start_time, :end_time, compare_type: :time)\n    #Ecto.Changeset<...>","ref":"FatEcto.Utils.Changeset.html#validate_start_before_end/4-examples"},{"type":"function","title":"FatEcto.Utils.Changeset.validate_start_before_or_equal_end/4","doc":"Validates that a start date/time is before or equal to an end date/time.","ref":"FatEcto.Utils.Changeset.html#validate_start_before_or_equal_end/4"},{"type":"function","title":"Parameters - FatEcto.Utils.Changeset.validate_start_before_or_equal_end/4","doc":"- `changeset`: The Ecto changeset.\n- `start_date_key`: The key for the start date/time.\n- `end_date_key`: The key for the end date/time.\n- `options`: Options to customize the error message or specify comparison type (`:time` or `:datetime`).","ref":"FatEcto.Utils.Changeset.html#validate_start_before_or_equal_end/4-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Changeset.validate_start_before_or_equal_end/4","doc":"iex> import Ecto.Changeset\n    iex> changeset = cast(%FatEcto.FatUser{}, %{start_time: ~T[10:00:00], end_time: ~T[10:00:00]}, [:start_time, :end_time])\n    iex> FatEcto.Utils.Changeset.validate_start_before_or_equal_end(changeset, :start_time, :end_time, compare_type: :time)\n    #Ecto.Changeset<...>","ref":"FatEcto.Utils.Changeset.html#validate_start_before_or_equal_end/4-examples"},{"type":"function","title":"FatEcto.Utils.Changeset.validate_xor_fields/4","doc":"Validates that only one of the specified XOR keys is present in the changeset.","ref":"FatEcto.Utils.Changeset.html#validate_xor_fields/4"},{"type":"function","title":"Parameters - FatEcto.Utils.Changeset.validate_xor_fields/4","doc":"- `changeset`: The Ecto changeset.\n- `record`: The original record (used to check if all XOR keys are empty).\n- `xor_keys`: A list of keys to validate as XOR.\n- `options`: Additional options (currently unused).","ref":"FatEcto.Utils.Changeset.html#validate_xor_fields/4-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Changeset.validate_xor_fields/4","doc":"iex> import Ecto.Changeset\n    iex> changeset = cast(%FatEcto.FatUser{}, %{field1: \"value1\", field2: \"value2\"}, [:field1, :field2])\n    iex> FatEcto.Utils.Changeset.validate_xor_fields(changeset, %FatEcto.FatUser{}, [:field1, :field2])\n    #Ecto.Changeset<...>","ref":"FatEcto.Utils.Changeset.html#validate_xor_fields/4-examples"},{"type":"module","title":"FatEcto.Utils.DateTime","doc":"Provides utility functions for parsing and converting date-time values.\n\nThis module handles parsing of date-time values from integers (Unix timestamps) and\nbinary strings (ISO 8601 format).","ref":"FatEcto.Utils.DateTime.html"},{"type":"function","title":"FatEcto.Utils.DateTime.parse/1","doc":"Parses a date-time value from an integer (Unix timestamp) or a binary string (ISO 8601 format).\n\nReturns `{:ok, DateTime.t()}` on success or `:error` on failure.","ref":"FatEcto.Utils.DateTime.html#parse/1"},{"type":"function","title":"FatEcto.Utils.DateTime.parse!/1","doc":"Parses a date-time value from an integer (Unix timestamp) or a binary string (ISO 8601 format).\n\nReturns a `DateTime.t()` on success or `nil` on failure.","ref":"FatEcto.Utils.DateTime.html#parse!/1"},{"type":"module","title":"FatEcto.Utils.Integer","doc":"Provides utility functions for parsing integers from strings.\n\nThis module handles parsing of integers from strings and ensures consistent return types.","ref":"FatEcto.Utils.Integer.html"},{"type":"function","title":"FatEcto.Utils.Integer.parse/1","doc":"Parses an integer from a string or returns the integer if already an integer.\n\nReturns `{:ok, integer}` on success or `{:error, nil}` on failure.","ref":"FatEcto.Utils.Integer.html#parse/1"},{"type":"function","title":"FatEcto.Utils.Integer.parse!/1","doc":"Parses an integer from a string or returns the integer if already an integer.\n\nReturns the parsed integer on success or `nil` on failure.","ref":"FatEcto.Utils.Integer.html#parse!/1"},{"type":"module","title":"FatEcto.Utils.Map","doc":"Provides utility functions for working with maps and structs.\n\nThis module includes functions for checking keys, values, and performing deep merges.","ref":"FatEcto.Utils.Map.html"},{"type":"function","title":"FatEcto.Utils.Map.contain_only_allowed_keys?/2","doc":"Checks if the map contains only the allowed keys.","ref":"FatEcto.Utils.Map.html#contain_only_allowed_keys?/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Map.contain_only_allowed_keys?/2","doc":"- `map`: The map to check.\n- `keys`: A list of allowed keys.","ref":"FatEcto.Utils.Map.html#contain_only_allowed_keys?/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Map.contain_only_allowed_keys?/2","doc":"iex> FatEcto.Utils.Map.contain_only_allowed_keys?(%{a: 1, b: 2}, [:a, :b])\n    true\n    iex> FatEcto.Utils.Map.contain_only_allowed_keys?(%{a: 1, c: 3}, [:a, :b])\n    false","ref":"FatEcto.Utils.Map.html#contain_only_allowed_keys?/2-examples"},{"type":"function","title":"FatEcto.Utils.Map.deep_merge/2","doc":"Deep merges two maps.","ref":"FatEcto.Utils.Map.html#deep_merge/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Map.deep_merge/2","doc":"- `left`: The first map.\n- `right`: The second map.","ref":"FatEcto.Utils.Map.html#deep_merge/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Map.deep_merge/2","doc":"iex> FatEcto.Utils.Map.deep_merge(%{a: %{b: 1}}, %{a: %{c: 2}})\n    %{a: %{b: 1, c: 2}}","ref":"FatEcto.Utils.Map.html#deep_merge/2-examples"},{"type":"function","title":"FatEcto.Utils.Map.get_keys_count/2","doc":"Counts the number of specified keys present in the map.","ref":"FatEcto.Utils.Map.html#get_keys_count/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Map.get_keys_count/2","doc":"- `map`: The map to check.\n- `keys`: A list of keys to count.","ref":"FatEcto.Utils.Map.html#get_keys_count/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Map.get_keys_count/2","doc":"iex> FatEcto.Utils.Map.get_keys_count(%{a: 1, b: 2}, [:a, :b, :c])\n    2","ref":"FatEcto.Utils.Map.html#get_keys_count/2-examples"},{"type":"function","title":"FatEcto.Utils.Map.has_all_keys?/2","doc":"Checks if all the specified keys are present in the map.","ref":"FatEcto.Utils.Map.html#has_all_keys?/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Map.has_all_keys?/2","doc":"- `map`: The map to check.\n- `keys`: A list of keys to check for.","ref":"FatEcto.Utils.Map.html#has_all_keys?/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Map.has_all_keys?/2","doc":"iex> FatEcto.Utils.Map.has_all_keys?(%{a: 1, b: 2}, [:a, :b])\n    true\n    iex> FatEcto.Utils.Map.has_all_keys?(%{a: 1}, [:a, :b])\n    false","ref":"FatEcto.Utils.Map.html#has_all_keys?/2-examples"},{"type":"function","title":"FatEcto.Utils.Map.has_all_keys_exclusive?/2","doc":"Checks if the map contains only the specified keys and no others.","ref":"FatEcto.Utils.Map.html#has_all_keys_exclusive?/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Map.has_all_keys_exclusive?/2","doc":"- `map`: The map to check.\n- `keys`: A list of allowed keys.","ref":"FatEcto.Utils.Map.html#has_all_keys_exclusive?/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Map.has_all_keys_exclusive?/2","doc":"iex> FatEcto.Utils.Map.has_all_keys_exclusive?(%{a: 1, b: 2}, [:a, :b])\n    true\n    iex> FatEcto.Utils.Map.has_all_keys_exclusive?(%{a: 1, c: 3}, [:a, :b])\n    false","ref":"FatEcto.Utils.Map.html#has_all_keys_exclusive?/2-examples"},{"type":"function","title":"FatEcto.Utils.Map.has_all_val_equal_to?/3","doc":"Checks if all the specified keys in the map have the given value.","ref":"FatEcto.Utils.Map.html#has_all_val_equal_to?/3"},{"type":"function","title":"Parameters - FatEcto.Utils.Map.has_all_val_equal_to?/3","doc":"- `map`: The map to check.\n- `keys`: A list of keys to check.\n- `equal_to`: The value to compare against.","ref":"FatEcto.Utils.Map.html#has_all_val_equal_to?/3-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Map.has_all_val_equal_to?/3","doc":"iex> FatEcto.Utils.Map.has_all_val_equal_to?(%{a: 1, b: 1}, [:a, :b], 1)\n    true\n    iex> FatEcto.Utils.Map.has_all_val_equal_to?(%{a: 1, b: 2}, [:a, :b], 1)\n    false","ref":"FatEcto.Utils.Map.html#has_all_val_equal_to?/3-examples"},{"type":"function","title":"FatEcto.Utils.Map.has_any_of_keys?/2","doc":"Checks if the map contains any of the specified keys.","ref":"FatEcto.Utils.Map.html#has_any_of_keys?/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Map.has_any_of_keys?/2","doc":"- `map`: The map to check.\n- `keys`: A list of keys to check for.","ref":"FatEcto.Utils.Map.html#has_any_of_keys?/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Map.has_any_of_keys?/2","doc":"iex> FatEcto.Utils.Map.has_any_of_keys?(%{a: 1}, [:a, :b])\n    true\n    iex> FatEcto.Utils.Map.has_any_of_keys?(%{c: 3}, [:a, :b])\n    false","ref":"FatEcto.Utils.Map.html#has_any_of_keys?/2-examples"},{"type":"module","title":"FatEcto.Utils.Network","doc":"Provides utility functions for working with network-related tasks.\n\nThis module includes functions for retrieving local network addresses.","ref":"FatEcto.Utils.Network.html"},{"type":"function","title":"FatEcto.Utils.Network.local_address/0","doc":"Retrieves the local network address.","ref":"FatEcto.Utils.Network.html#local_address/0"},{"type":"function","title":"Examples - FatEcto.Utils.Network.local_address/0","doc":"iex> FatEcto.Utils.Network.local_address()\n    \"192.168.1.100\"","ref":"FatEcto.Utils.Network.html#local_address/0-examples"},{"type":"function","title":"Returns - FatEcto.Utils.Network.local_address/0","doc":"- A string representing the local network address, or `nil` if the address cannot be determined.","ref":"FatEcto.Utils.Network.html#local_address/0-returns"},{"type":"module","title":"FatEcto.Utils.String","doc":"Provides utility functions for generating random strings.\n\nThis module includes functions for generating random strings of specified lengths\nand from specific character sets.","ref":"FatEcto.Utils.String.html"},{"type":"function","title":"FatEcto.Utils.String.random/1","doc":"Generates a random string of the specified length.","ref":"FatEcto.Utils.String.html#random/1"},{"type":"function","title":"Parameters - FatEcto.Utils.String.random/1","doc":"- `length`: The length of the random string (default: 8).","ref":"FatEcto.Utils.String.html#random/1-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.String.random/1","doc":"iex> FatEcto.Utils.String.random()\n    \"aB3dEfG1\"\n\n    iex> FatEcto.Utils.String.random(12)\n    \"xYz1aB2cD3eF\"","ref":"FatEcto.Utils.String.html#random/1-examples"},{"type":"function","title":"FatEcto.Utils.String.random_of/2","doc":"Generates a random string of the specified length using a custom character set.","ref":"FatEcto.Utils.String.html#random_of/2"},{"type":"function","title":"Parameters - FatEcto.Utils.String.random_of/2","doc":"- `length`: The length of the random string.\n- `char_set`: A list of characters to use for generating the string (default: A-Z).","ref":"FatEcto.Utils.String.html#random_of/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.String.random_of/2","doc":"iex> FatEcto.Utils.String.random_of(10)\n    \"ABCDEFGHIJ\"\n\n    iex> FatEcto.Utils.String.random_of(5, [\"a\", \"b\", \"c\"])\n    \"abcba\"","ref":"FatEcto.Utils.String.html#random_of/2-examples"},{"type":"module","title":"FatEcto.Utils.Table","doc":"Provides utility functions for working with database tables.\n\nThis module includes functions to generate SQL queries for common table operations,\nsuch as resetting the ID sequence of a table.","ref":"FatEcto.Utils.Table.html"},{"type":"function","title":"FatEcto.Utils.Table.reset_id_seq_query/2","doc":"Generates a SQL query to reset the ID sequence of a given table.","ref":"FatEcto.Utils.Table.html#reset_id_seq_query/2"},{"type":"function","title":"Parameters - FatEcto.Utils.Table.reset_id_seq_query/2","doc":"- `table`: The name of the table as a string or atom.\n  - `id`: The name of the ID column as a string or atom. Defaults to `\"id\"`.","ref":"FatEcto.Utils.Table.html#reset_id_seq_query/2-parameters"},{"type":"function","title":"Examples - FatEcto.Utils.Table.reset_id_seq_query/2","doc":"iex> FatEcto.Utils.Table.reset_id_seq_query(\"users\")\n    \"SELECT setval('users_id_seq', (SELECT MAX(id) from \"users\"));\"\n\n    iex> FatEcto.Utils.Table.reset_id_seq_query(\"posts\", \"post_id\")\n    \"SELECT setval('posts_post_id_seq', (SELECT MAX(post_id) from \"posts\"));\"","ref":"FatEcto.Utils.Table.html#reset_id_seq_query/2-examples"},{"type":"function","title":"Notes - FatEcto.Utils.Table.reset_id_seq_query/2","doc":"- Ensure that the table has no records or that the sequence reset is appropriate for your use case.\n- This function only generates the SQL query. You need to execute it using your database driver or migration tool.","ref":"FatEcto.Utils.Table.html#reset_id_seq_query/2-notes"},{"type":"module","title":"FatEcto.Utils.UUID","doc":"Provides utility functions for working with UUIDs.\n\nThis module includes functions to validate and parse UUIDs from a map of parameters.","ref":"FatEcto.Utils.UUID.html"},{"type":"function","title":"FatEcto.Utils.UUID.parse/3","doc":"Parses and validates UUIDs in a map of parameters.\n\nThis function takes a map of parameters, a list of fields that should contain UUIDs,\nand an optional list of options. It returns a tuple containing a list of invalid UUID fields\nand a map of valid parameters.","ref":"FatEcto.Utils.UUID.html#parse/3"},{"type":"function","title":"Parameters - FatEcto.Utils.UUID.parse/3","doc":"- `params`: A map of parameters.\n  - `list_of_uuid_fields`: A list of fields (keys) in the map that should contain UUIDs.\n  - `options`: A keyword list of options. Supported options:\n    - `:valid_values`: A list of values that are considered valid without UUID validation.","ref":"FatEcto.Utils.UUID.html#parse/3-parameters"},{"type":"function","title":"Returns - FatEcto.Utils.UUID.parse/3","doc":"- `{invalid_uuids, valid_params}`: A tuple where:\n    - `invalid_uuids` is a list of fields that contain invalid UUIDs.\n    - `valid_params` is a map of parameters with valid UUIDs.","ref":"FatEcto.Utils.UUID.html#parse/3-returns"},{"type":"function","title":"Examples - FatEcto.Utils.UUID.parse/3","doc":"iex> params = %{user_id: \"123e4567-e89b-12d3-a456-426614174000\", post_id: \"invalid-uuid\"}\n    iex> FatEcto.Utils.UUID.parse(params, [:user_id, :post_id])\n    {[:post_id], %{user_id: \"123e4567-e89b-12d3-a456-426614174000\"}}\n\n    iex> params = %{user_id: \"123e4567-e89b-12d3-a456-426614174000\", role: \"admin\"}\n    iex> FatEcto.Utils.UUID.parse(params, [:user_id], valid_values: [\"admin\"])\n    {[], %{user_id: \"123e4567-e89b-12d3-a456-426614174000\", role: \"admin\"}}","ref":"FatEcto.Utils.UUID.html#parse/3-examples"},{"type":"extras","title":"FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"# FatEcto: Supercharge Your Ecto Queries with Ease! 🚀\n\n[![Build Status](https://github.com/tanweerdev/fat_ecto/actions/workflows/fat_ecto.yml/badge.svg)](https://github.com/tanweerdev/fat_ecto/actions)\n[![Coverage Status](https://coveralls.io/repos/github/tanweerdev/fat_ecto/badge.svg)](https://coveralls.io/github/tanweerdev/fat_ecto)\n[![hex.pm version](https://img.shields.io/hexpm/v/fat_ecto.svg)](https://hex.pm/packages/fat_ecto)\n[![hex.pm downloads](https://img.shields.io/hexpm/dt/fat_ecto.svg)](https://hex.pm/packages/fat_ecto)\n[![hex.pm license](https://img.shields.io/hexpm/l/fat_ecto.svg)](https://github.com/tanweerdev/fat_ecto/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/tanweerdev/fat_ecto.svg)](https://github.com/tanweerdev/fat_ecto/commits/master)\n\n---","ref":"readme.html"},{"type":"extras","title":"Description - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"FatEcto is an Elixir package designed to make your life easier when working with Ecto. It simplifies query building, filtering, sorting, pagination, and data sanitization—so you can focus on what truly matters: building amazing applications. With FatEcto, writing complex queries becomes effortless, flexible, and powerful! 💪\n\n---","ref":"readme.html#description"},{"type":"extras","title":"Installation - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"Getting started is simple! Add `fat_ecto` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    # Check https://hexdocs.pm/fat_ecto for the latest version\n    {:fat_ecto, \"~> 1.0.0\"}\n  ]\nend\n```\n\nThen, run `mix deps.get` to install the package.\n\n---","ref":"readme.html#installation"},{"type":"extras","title":"Features & Modules - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"","ref":"readme.html#features-modules"},{"type":"extras","title":"🛠 FatEcto.Dynamics.FatBuildable – Dynamic Filtering Made Easy - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"Tired of writing repetitive query filters? The `Whereable` module lets you dynamically filter records using flexible conditions passed from your web or mobile clients—with little to no effort! And the best part? You stay in control. 🚀\n\n#### Usage\n\n```elixir\ndefmodule FatEcto.Dynamics.MyApp.HospitalFilter do\n  use FatEcto.Dynamics.FatBuildable,\n    filterable_fields: %{\n      \"id\" => [\"$EQUAL\", \"$NOT_EQUAL\"]\n    },\n    overrideable_fields: [\"name\", \"phone\"],\n    ignoreable_fields_values: %{\n      \"name\" => [\"%%\", \"\", [], nil],\n      \"phone\" => [\"%%\", \"\", [], nil]\n    }\n\n  import Ecto.Query\n\n  @impl true\n  # You can implement override_whereable for your custom filters\n  def override_whereable(_dynamics, \"name\", \"$ILIKE\", value) do\n    dynamic([r], ilike(fragment(\"(?)::TEXT\", r.name), ^value))\n  end\n\n  def override_whereable(dynamics, _, _, _), do: dynamics\n\n  # You can do some custom processing if needed eg\n  def after_whereable(dynamics) do\n    if dynamics, do: dynamics, else: true\n  end\nend\n```\n\n---\n\n#### Example Usage\n\nHere are some practical examples of how to use `FatEcto.Dynamics.MyApp.HospitalFilter` to dynamically build queries:\n\n##### Example 1: Basic Filtering by ID\n\n```elixir\n# Filter hospitals with ID equal to 1\nparams = %{\"id\" => %{\"$EQUAL\" => 1}}\ndynamics = FatEcto.Dynamics.MyApp.HospitalFilter.build(params)\n\n# Use the dynamics in a query\nimport Ecto.Query\nquery = where(FatEcto.FatHospital, ^dynamics)\n\n# Resulting query:\n# from(h in FatEcto.FatHospital, where: h.id == 1)\n```\n\n##### Example 2: Case-Insensitive Name Search\n\n```elixir\n# Filter hospitals with names containing \"St. Mary\"\nparams = %{\"name\" => %{\"$ILIKE\" => \"%St. Mary%\"}}\ndynamics = FatEcto.Dynamics.MyApp.HospitalFilter.build(params)\n\n# Use the dynamics in a query\nimport Ecto.Query\nquery = where(FatEcto.FatHospital, ^dynamics)\n\n# Resulting query:\n# from(h in FatEcto.FatHospital, where: ilike(fragment(\"(?)::TEXT\", h.name), ^\"%St. Mary%\"))\n```\n\n##### Example 3: Combining Multiple Filters\n\n```elixir\n# Filter hospitals with ID not equal to 2 AND name containing \"General\"\nparams = %{\n  \"id\" => %{\"$NOT_EQUAL\" => 2},\n  \"name\" => %{\"$ILIKE\" => \"%General%\"}\n}\ndynamics = FatEcto.Dynamics.MyApp.HospitalFilter.build(params)\n\n# Use the dynamics in a query\nimport Ecto.Query\nquery = where(FatEcto.FatHospital, ^dynamics)\n\n# Resulting query:\n# from(h in FatEcto.FatHospital, where: h.id != 2 and ilike(fragment(\"(?)::TEXT\", h.name), ^\"%General%\"))\n```\n\n##### Example 4: Ignoring Empty or Invalid Values\n\n```elixir\n# Filter hospitals with a name, but ignore empty or invalid values\nparams = %{\"name\" => %{\"$ILIKE\" => \"%%\"}}  # Empty value is ignored\ndynamics = FatEcto.Dynamics.MyApp.HospitalFilter.build(params)\n\n# Use the dynamics in a query\nimport Ecto.Query\nquery = where(FatEcto.FatHospital, ^dynamics)\n\n# Resulting query:\n# from(h in FatEcto.FatHospital)  # No filtering applied for name\n```\n\n##### Example 5: Even Complex Nested conditions\n\n```elixir\n# Filter hospitals with a name, but ignore empty or invalid values\nparams = %{\n  \"$OR\" => [\n    %{\n      \"name\" => %{\"$ILIKE\" => \"%John%\"},\n      \"$OR\" => %{\"rating\" => %{\"$GT\" => 18}, \"location\" => \"New York\"}\n    },\n    %{\n      \"start_date\" => \"2023-01-01\",\n      \"$AND\" => [\n        %{\"rating\" => %{\"$GT\" => 4}},\n        %{\"email\" => \"fat_ecto@example.com\"}\n      ]\n    }\n  ]\n}\n\ndynamics = DoctorFilter.build(params)\n\n# Resulting dynamic:\ndynamic(\n  [q],\n  ((q.location == ^\"New York\" or q.rating > ^18) and ilike(fragment(\"(?)::TEXT\", q.name), ^\"%John%\")) or\n    (q.rating > ^4 and q.email == ^\"fat_ecto@example.com\" and q.start_date == ^\"2023-01-01\")\n)\n\n# You can now apply the result on where just like above examples\n```\n\n---","ref":"readme.html#fatecto-dynamics-fatbuildable-dynamic-filtering-made-easy"},{"type":"extras","title":"🔄 FatEcto.FatSortable – Effortless Sorting - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"Sorting should be simple—and with `Sortable`, it is! Your frontend can send sorting parameters, and FatEcto will seamlessly generate the right sorting queries, allowing you to build powerful, customizable sorting logic without breaking a sweat. 😎\n\n#### Usage of FatSortable\n\n```elixir\ndefmodule Fat.SortQuery do\n  import Ecto.Query\n  use FatEcto.FatSortable,\n    sortable_fields: %{\"id\" => \"$ASC\", \"name\" => [\"$ASC\", \"$DESC\"]},\n    overrideable_fields: [\"custom_field\"]\n\n  @impl true\n  def override_sortable(query, field, operator) do\n    case {field, operator} do\n      {\"custom_field\", \"$ASC\"} ->\n        from(q in query, order_by: [asc: fragment(\"?::jsonb->>'custom_field'\", q)])\n      _ ->\n        query\n    end\n  end\nend\n```\n\n---","ref":"readme.html#fatecto-fatsortable-effortless-sorting"},{"type":"extras","title":"📌 FatEcto.FatPaginator – Paginate Like a Pro - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"No more hassle with pagination! FatPaginator helps you paginate Ecto queries efficiently, keeping your APIs snappy and responsive.\n\n#### Usage of FatPaginator\n\n```elixir\ndefmodule Fat.MyPaginator do\n  use FatEcto.FatPaginator, repo: Fat.Repo\n  # Add custom pagination functions here\nend\n```\n\n---","ref":"readme.html#fatecto-fatpaginator-paginate-like-a-pro"},{"type":"extras","title":"🔍 FatEcto.FatDataSanitizer – Clean & Structured Data - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"Messy data? Not anymore! `DataSanitizer` helps you sanitize records and transform them into structured, clean views effortlessly. Keep your data tidy and consistent. 🎯\n\n#### Usage of FatDataSanitizer\n\n```elixir\ndefmodule Fat.MySanitizer do\n  use FatEcto.FatDataSanitizer\n  # Define your custom sanitization functions here\nend\n```\n\n---","ref":"readme.html#fatecto-fatdatasanitizer-clean-structured-data"},{"type":"extras","title":"⚡ FatEcto Utilities – Small Helpers, Big Impact - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"FatEcto also comes with a set of handy utility functions to streamline your workflow:\n\n```elixir\n# Check if a map contains all required keys\nFatEcto.Utils.Map.has_all_keys?(%{a: 1, b: 2}, [:a, :b])\n\n# Ensure a map contains only allowed keys\nFatEcto.Utils.Map.contain_only_allowed_keys?(%{a: 1, c: 3}, [:a, :b])\n```\n\n---","ref":"readme.html#fatecto-utilities-small-helpers-big-impact"},{"type":"extras","title":"🚀 Contributing - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"We love contributions! If you’d like to improve FatEcto, submit an issue or pull request. Let’s build something amazing together! 🔥\n\n---","ref":"readme.html#contributing"},{"type":"extras","title":"📜 License - FatEcto: Supercharge Your Ecto Queries with Ease! 🚀","doc":"FatEcto is released under the MIT License.\n\n📖 See the full documentation at [HexDocs](https://hexdocs.pm/fat_ecto/) for more details.","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}